declare type IGenericField<T, Co> = Co | {
    type: Co;
    default?: T;
    optional?: boolean;
    description?: string;
};
declare type IValidTypes = string | number | boolean;
declare type ITestField<T1 extends IValidTypes, T2 extends IValidTypes, R> = T1 extends T2 ? R : never;
export declare type IField<F extends IValidTypes> = ITestField<F, string, IGenericField<string, StringConstructor>> | ITestField<F, number, IGenericField<number, NumberConstructor>> | ITestField<F, boolean, IGenericField<boolean, BooleanConstructor>>;
export declare type IConfigInterface = {
    [category: string]: {
        [field: string]: IValidTypes | undefined;
    };
};
export declare const Optional: unique symbol;
export declare type IOptional<T extends IConfigInterface> = {
    [category in keyof T]: {
        [Optional]?: boolean;
    };
};
export declare type IFields<T extends IConfigInterface> = {
    [category in keyof T]: {
        [field in keyof T[category]]: IField<T[category][field]>;
    };
};
export declare type IConfig<T extends IConfigInterface> = IFields<T> | IOptional<T>;
declare function parse<T extends IConfigInterface>(config: IConfig<T>, path?: string): T;
declare function parseAsync<T extends IConfigInterface>(config: IConfig<T>, path: string): Promise<T>;
declare function getOptionsHelp(config: IConfig<any>): {
    ini: string;
    docker: string;
};
declare function printHelp(config: IConfig<any>): void;
export { parse, parseAsync, getOptionsHelp, printHelp };
