"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ini = require("ini");
const fs = require("fs");
exports.Optional = Symbol("optional");
function parseData(config, rawdata) {
    const data = rawdata ? ini.parse(rawdata) : {};
    let result = {};
    let errors = [];
    for (let C in config) {
        if (config[C][exports.Optional]) {
            if (!data[C] &&
                !Object.keys(process.env).find((key) => key.startsWith(`${C.toUpperCase}_`))) {
                continue;
            }
        }
        result[C] = {};
        for (let F in config[C]) {
            let gfieldConfig = config[C][F];
            let fieldConfig = (gfieldConfig === Number ||
                gfieldConfig === String ||
                gfieldConfig === Boolean
                ? { type: gfieldConfig }
                : gfieldConfig);
            const envname = `${C.toUpperCase()}_${F.toUpperCase()}`;
            let value = (data[C] || {})[F] || process.env[envname] || fieldConfig.default;
            if (value !== undefined && value !== null) {
                switch (fieldConfig.type) {
                    case Number:
                        value = Number(value);
                        break;
                    case String:
                        value = String(value);
                        break;
                    case Boolean:
                        value = value === "true";
                        break;
                }
            }
            else if (!fieldConfig.optional) {
                errors.push(`[${C}][${F}] Cannot be undefined!`);
            }
            result[C][F] = value;
        }
    }
    if (errors.length > 0)
        throw new Error("There are errors: \n" + errors.join("\n"));
    return result;
}
function parse(config, path) {
    let data;
    if (path)
        data = fs.existsSync(path)
            ? fs.readFileSync(path).toString("utf8")
            : undefined;
    return parseData(config, data);
}
exports.parse = parse;
async function parseAsync(config, path) {
    let data;
    if (path && (await new Promise((yes) => fs.exists(path, yes)))) {
        data = await new Promise((yes, no) => fs.readFile(path, (err, data) => (err ? no(err) : yes(data)))).then((res) => res.toString("utf8"));
    }
    return parseData(config, data);
}
exports.parseAsync = parseAsync;
const typeNameMap = new Map();
typeNameMap.set(String, "string");
typeNameMap.set(Number, "number");
typeNameMap.set(Boolean, "boolean");
function getOptionsHelp(config) {
    let ini = [];
    let docker = [];
    for (const category in config) {
        ini.push(`[${category}]`);
        for (const key in config[category]) {
            let entry = config[category][key];
            let field = (entry === Number || entry === String || entry === Boolean
                ? { type: entry }
                : entry);
            let typeName = typeNameMap.get(field.type);
            let optional = !!field.optional;
            let description = field.description || "";
            let def = field.default ? ` (default: ${field.default})` : "";
            ini.push(`${key}=<${optional ? "?" : ""}${typeName}>${def} ${description}`);
            docker.push(`${category.toUpperCase()}_${key.toUpperCase()}=<${optional ? "?" : ""}${typeName}>${def} ${description}`);
        }
    }
    return { ini: ini.join("\n"), docker: docker.join("\n") };
}
exports.getOptionsHelp = getOptionsHelp;
function printHelp(config) {
    const help = getOptionsHelp(config);
    console.log("ini config:");
    console.log(help.ini);
    console.log("");
    console.log("Docker config:");
    console.log(help.docker);
}
exports.printHelp = printHelp;
//# sourceMappingURL=index.js.map